; ====================================
;  HDClean v1.0 (c) 2003 by key2k3
; ====================================
;  When started with no parameters, it creates an entry in the 'RunOnce'-
;  registry key, pointing to itself with parameter '-run'. The system will
;  be rebooted immediately, if the right mousebutton is pressed during
;  start.
;  Parameter '-run' causes HDClean to start Scandisk in automatic noninter-
;  active mode which will terminate Scandisk after scanning all fixed
;  drives. HDClean waits for this event to occur and then launches DeFrag
;  also in automatic mode. If DeFrag finished its work, the system will be
;  shutdown.
;  The right mousebutton can also be held down while aborting Scandisk or
;  DeFrag, which will stop HDClean and let windows start as usual. 
;  
;  TECHNICAL NOTE:
;   If DeFrag encounters an error, it will display the well-known error mes-
;   sages (DEFRAG005 or DEFRAG006) even if noninteractive mode is chosen via
;   command-line. Therefore, HDClean looks for any child-popup-window created
;   by the DeFrag window (via GetLastActivePopup) and simulates pressing
;   the ENTER-Key if a popup is detected. Since DeFrag won't come up with
;   anything else than the mentioned error-message and the "OK" Button is
;   the default, pressing ENTER will exit DeFrag, so HDClean can start over
;   with ScanDisk.
;   
;  FOR MORE INFO, VISIT http://www.mvps.org/serenitymacros/winprogs.html
; ====================================


.586
.model flat, stdcall  ; 32 bit memory model
option casemap :none  ; case sensitive

include \MASM32\INCLUDE\windows.inc
include \MASM32\INCLUDE\kernel32.inc
include \MASM32\INCLUDE\user32.inc
include \MASM32\INCLUDE\shell32.inc
include \MASM32\INCLUDE\advapi32.inc

includelib \MASM32\LIB\kernel32.lib
includelib \MASM32\LIB\user32.lib
includelib \MASM32\LIB\shell32.lib
includelib \MASM32\LIB\advapi32.lib

.data?

 hKey       dd ?
 hPID       dd ?
 hWndDfr    dd ?
 dwDisp     dd ?
 dwScan     dd ?
 dwLen      dd ?
 szBuff     db 300 DUP(?)

.data

 szRegVal   db '"', 300 DUP(0)
 szRegName  db 'HDClean StartUp', 0
 szRunOnce  db 'Software\Microsoft\Windows\CurrentVersion\RunOnce\', 0

 szOpen     db "open", 0
 szScanCmd  db "scandskw.exe", 0
 szScanArgs db "/A /N", 0
 szScanClas db "ScanDskWDlgClass", 0
 szDefCmd   db "defrag.exe", 0
 szDefArgs  db "/all /F /concise /noprompt", 0
 szDefClass db "MSDefragWClass1", 0

 szErrTxt   db "An error occured. Operation canceled!", 0
 szErrTit   db "Fatal Error", 0

.code

start:
    invoke  GetModuleFileName, NULL, ADDR szBuff, SIZEOF szBuff
    mov     dwLen, eax
    invoke  GetCommandLine
    cmp     byte ptr [eax], '"'
    add     eax, dwLen
    je      NoQuotes
    inc     eax
    inc     eax

  NoQuotes:
    cmp     word ptr [eax], 20h
    je      NoArgs
    cmp     dword ptr [eax], 75722D20h
    jne     Err
    cmp     word ptr [eax+4], 6Eh
    jne     Err
    
  Run:  
    invoke  GetAsyncKeyState, VK_RBUTTON
    cmp     eax, 1
    ja      Xit
    
    invoke  ShellExecute, NULL, ADDR szOpen, ADDR szScanCmd, ADDR szScanArgs, NULL, SW_SHOWNORMAL
    invoke  Sleep, 7000
    invoke  FindWindow, ADDR szScanClas, NULL
    invoke  GetWindowThreadProcessId, eax, ADDR hPID
    invoke  OpenProcess, PROCESS_ALL_ACCESS, FALSE, hPID
    push    eax
    invoke  WaitForSingleObject, eax, INFINITE
    pop     eax
    invoke  CloseHandle, eax

    invoke  GetAsyncKeyState, VK_RBUTTON
    cmp     eax, 1
    ja      Xit

    invoke  ShellExecute, NULL, ADDR szOpen, ADDR szDefCmd, ADDR szDefArgs, NULL, SW_SHOWNORMAL
    invoke  Sleep, 7000
    invoke  FindWindow, ADDR szDefClass, NULL
    mov     hWndDfr, eax
    invoke  GetWindowThreadProcessId, eax, ADDR hPID
    invoke  OpenProcess, PROCESS_ALL_ACCESS, FALSE, hPID
    push    eax

  DfrWait:
    invoke  WaitForSingleObject, [esp+4], 50000
    cmp     eax, WAIT_TIMEOUT
    jne     DfrFinished
    invoke  GetLastActivePopup, hWndDfr
    cmp     eax, hWndDfr
    je      DfrWait

    invoke  MapVirtualKey, VK_RETURN, 0
    mov     dwScan, eax
    invoke  keybd_event, VK_RETURN, dwScan, NULL, NULL
    invoke  Sleep, 20                             
    invoke  keybd_event, VK_RETURN, dwScan, KEYEVENTF_KEYUP, NULL
    
    invoke  WaitForSingleObject, [esp+4], INFINITE
    call    CloseHandle
    jmp     Run

  DfrFinished:
    call    CloseHandle

    invoke  GetAsyncKeyState, VK_RBUTTON
    cmp     eax, 1
    ja      Xit

    invoke  ExitWindowsEx, EWX_SHUTDOWN, NULL
    invoke  Sleep, 2000
    jmp     Xit
    
  NoArgs:
    invoke  RegCreateKeyEx, HKEY_LOCAL_MACHINE, ADDR szRunOnce, NULL, NULL, REG_OPTION_NON_VOLATILE,
                            KEY_ALL_ACCESS, NULL, ADDR hKey, ADDR dwDisp
    test    eax, eax
    jnz     Err
    invoke  GetModuleFileName, NULL, ADDR (szRegVal+1), (SIZEOF szRegVal)-8
    lea     edi, szRegVal+1
    add     edi, eax
    mov     word ptr [edi], ' "'
    mov     dword ptr [edi+2], 'nur-'
    add     eax, 8
    invoke  RegSetValueEx, hKey, ADDR szRegName, NULL, REG_SZ, ADDR szRegVal, eax
    push    eax
    invoke  RegCloseKey, hKey
    pop     eax
    test    eax, eax
    jnz     Err

    invoke  GetAsyncKeyState, VK_RBUTTON
    cmp     eax, 1
    jna     Xit

    invoke  ExitWindowsEx, EWX_REBOOT, NULL
    jmp Xit

  Err:
    invoke  MessageBox, NULL, ADDR szErrTxt, ADDR szErrTit, MB_ICONERROR or MB_OK

  Xit:
    invoke  ExitProcess, 1
end start
